@@ -18,7 +18,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:Public);
 
-$VERSION = '1.20';
+$VERSION = '1.26';
 
 sub SetGeoValues($$;$);
 
@@ -46,6 +46,7 @@ my %xmlTag = (
     # XML containers (fix is reset at the opening tag of these properties)
     wpt         => '',          # GPX
     trkpt       => '',          # GPX
+    rtept       => '',          # GPX
     trackpoint  => '',          # Garmin
     placemark   => '',          # KML
 );
@@ -87,23 +88,39 @@ sub LoadTrackLog($$;$)
     }
     # add data to existing track
     my $geotag = $exifTool->GetNewValues('Geotag') || { };
-    if ($val =~ /(\x0d\x0a|\x0d|\x0a)/) {
-        # $val is track log data
+    my $format = '';
+    # is $val track log data?
+    if ($val =~ /^(\xef\xbb\xbf)?<(\?xml|gpx)\s/) {
+        $format = 'XML';
+        $/ = '>';   # set input record separator to '>' for XML/GPX data
+    } elsif ($val =~ /(\x0d\x0a|\x0d|\x0a)/) {
         $/ = $1;
-        $raf = new File::RandomAccess(\$val);
-        $from = 'data';
     } else {
         # $val is track file name
         open EXIFTOOL_TRKFILE, $val or return "Error opening GPS file '$val'";
         $raf = new File::RandomAccess(\*EXIFTOOL_TRKFILE);
-        unless ($raf->Read($_, 256) and /(\x0d\x0a|\x0d|\x0a)/) {
+        unless ($raf->Read($_, 256)) {
+            close EXIFTOOL_TRKFILE;
+            return "Empty track file '$val'";
+        }
+        # look for XML or GPX header (might as well allow UTF-8 BOM)
+        if (/^(\xef\xbb\xbf)?<(\?xml|gpx)\s/) {
+            $format = 'XML';
+            $/ = '>';   # set input record separator to '>' for XML/GPX data
+        } elsif (/(\x0d\x0a|\x0d|\x0a)/) {
+            $/ = $1;
+        } else {
             close EXIFTOOL_TRKFILE;
             return "Invalid track file '$val'";
         }
-        $/ = $1;
         $raf->Seek(0,0);
         $from = "file '$val'";
     }
+    unless ($from) {
+        # set up RAF for reading log file in memory
+        $raf = new File::RandomAccess(\$val);
+        $from = 'data';
+    }
     # initialize track points lookup
     my $points = $$geotag{Points};
     $points or $points = $$geotag{Points} = { };
@@ -117,7 +134,6 @@ sub LoadTrackLog($$;$)
     my $numPoints = 0;
     my $skipped = 0;
     my $lastSecs = 0;
-    my $format = '';
     my $fix = { };
     for (;;) {
         $raf->ReadLine($_) or last;
@@ -192,7 +208,7 @@ sub LoadTrackLog($$;$)
                         $td = 1;
                     }
                     # validate and store GPS fix
-                    if ($$fix{lat} and $$fix{lon} and $$fix{'time'} and
+                    if (defined $$fix{lat} and defined $$fix{lon} and $$fix{'time'} and
                         $$fix{lat} =~ /^[+-]?\d+\.?\d*/ and
                         $$fix{lon} =~ /^[+-]?\d+\.?\d*/ and
                         $$fix{'time'} =~ /^(\d{4})-(\d+)-(\d+)T(\d+):(\d+):(\d+)(\.\d+)?(.*)/)
@@ -201,13 +217,15 @@ sub LoadTrackLog($$;$)
                         $time += $7 if $7;  # add fractional seconds
                         my $tz = $8;
                         # adjust for time zone (otherwise assume UTC)
-                        if ($tz =~ /^([-+])(\d+):(\d{2})\b/) {
-                            $tz = ($2 * 60 + $3) * 60;
+                        # - allow timezone of +-HH:MM, +-H:MM, +-HHMM or +-HH since
+                        #   the spec is unclear about timezone format
+                        if ($tz =~ /^([-+])(\d+):(\d{2})\b/ or $tz =~ /^([-+])(\d{2})(\d{2})?\b/) {
+                            $tz = ($2 * 60 + ($3 || 0)) * 60;
                             $tz *= -1 if $1 eq '+'; # opposite sign to change back to UTC
                             $time += $tz;
                         }
                         # validate altitude
-                        undef $$fix{alt} if $$fix{alt} and $$fix{alt} !~ /^[+-]?\d+\.?\d*/;
+                        undef $$fix{alt} if defined $$fix{alt} and $$fix{alt} !~ /^[+-]?\d+\.?\d*/;
                         $isDate = 1;
                         $canCut= 1 if defined $$fix{pdop} or defined $$fix{hdop} or defined $$fix{nsats};
                         $$points{$time} = $fix;
@@ -260,6 +278,7 @@ sub LoadTrackLog($$;$)
             $secs = (($9 * 60) + $10) * 60 + $11;
             $secs += $12 if $12;    # add fractional seconds
             if (defined $15) {
+                next if $13 > 31 or $14 > 12 or $15 > 99;   # validate day/month/year
                 # optional date is available in PMGNTRK sentence
                 my $year = $15 + ($15 >= 70 ? 1900 : 2000);
                 $date = Time::Local::timegm(0,0,0,$13,$14-1,$year-1900);
@@ -271,6 +290,7 @@ sub LoadTrackLog($$;$)
             #  $GPRMC,092204.999,A,4250.5589,S,14718.5084,E,0.00,89.68,211200,,*25
             #  $GPRMC,hhmmss.sss,A/V,ddmm.mmmm,N/S,ddmmm.mmmm,E/W,spd(knots),dir(deg),DDMMYY,,*cs
             /^\$GPRMC,(\d{2})(\d{2})(\d+)(\.\d+)?,A,(\d+)(\d{2}\.\d+),([NS]),(\d+)(\d{2}\.\d+),([EW]),[^,]*,[^,]*,(\d{2})(\d{2})(\d+)/ or next;
+            next if $11 > 31 or $12 > 12 or $13 > 99;   # validate day/month/year
             $fix{lat} = ($5 + $6/60) * ($7 eq 'N' ? 1 : -1);
             $fix{lon} = ($8 + $9/60) * ($10 eq 'E' ? 1 : -1);
             my $year = $13 + ($13 >= 70 ? 1900 : 2000);
@@ -570,7 +590,7 @@ sub SetGeoValues($$;$)
             ($year,$mon,$day) = (1970,1,2);
             $noDate = 1;
         } else {
-            $err = 'Invalid date/time (use YYYY:MM:DD HH:MM:SS[.SS][+/-HH:MM|Z])';
+            $err = 'Invalid date/time (use YYYY:mm:dd HH:MM:SS[.ss][+/-HH:MM|Z])';
             last;
         }
         if ($tz) {
@@ -668,9 +688,9 @@ sub SetGeoValues($$;$)
         # write GPSDateStamp if date included in track log, otherwise delete it
         $gpsDate = sprintf('%.2d:%.2d:%.2d', $t[5]+1900, $t[4]+1, $t[3]) unless $noDate;
         # write GPSAltitude tags if altitude included in track log, otherwise delete them
-        if ($$fix{alt}) {
+        if (defined $$fix{alt}) {
             $gpsAlt = abs $$fix{alt};
-            $gpsAltRef = ($$fix{alt} > 0 ? 0 : 1);
+            $gpsAltRef = ($$fix{alt} < 0 ? 1 : 0);
         }
         # set new GPS tag values (EXIF, or XMP if write group is 'xmp')
         my ($xmp, $exif, @r);
@@ -864,7 +884,7 @@ in the tag name documentation).
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
