@@ -15,30 +15,43 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.02';
+$VERSION = '1.09';
 
 # enable MWG strict mode by default
 # (causes non-standard EXIF, IPTC and XMP to be ignored)
 $Image::ExifTool::MWG::strict = 1 unless defined $Image::ExifTool::MWG::strict;
 
 sub RecoverTruncatedIPTC($$$);
+sub ListToString($);
+sub StringToList($$);
+sub OverwriteStringList($$$$);
 
 # MWG Composite tags
 %Image::ExifTool::MWG::Composite = (
     GROUPS => { 0 => 'Composite', 1 => 'MWG', 2 => 'Image' },
     VARS => { NO_ID => 1 },
     NOTES => q{
-        This table lists special Composite tags which are used to access other tags
-        based on the Metadata Working Group 1.01 recommendations.  These tags are
-        only accessible when the MWG module is loaded (via C<-use MWG> on the
-        command line, or C<use Image::ExifTool::MWG> in the API).  When reading, the
-        value of each MWG tag is B<Derived From> the specified tags based on the MWG
-        guidelines.  When writing, the value of the IPTCDigest tag is updated
-        automatically when the IPTC is changed if either the IPTCDigest tag didn't
-        exist beforehand or its value agreed with the original IPTC digest
-        (indicating that the XMP is synchronized with the IPTC).  IPTC information
-        is written only if the original file contained IPTC.  For more information
-        about the MWG recommendation, see L<http://www.metadataworkinggroup.org/>.
+        The Metadata Working Group (MWG) recommendations provide a set of rules to
+        allow certain overlapping EXIF, IPTC and XMP tags to be reconciled when
+        reading, and synchronized when writing.  The ExifTool MWG module is designed
+        to aid in the implementation of these recommendations.  (See
+        L<http://www.metadataworkinggroup.org/> for the complete MWG technical
+        specifications.)
+
+        The table below lists special Composite tags which are used to access other
+        tags based on the MWG 2.0 recommendations.  These tags are only accessible
+        when the MWG module is loaded.  The MWG module is loaded automatically by
+        the exiftool application if MWG is specified as a group for any tag on the
+        command line, or manually with the C<-use MWG> option.  Via the API, the MWG
+        module is loaded with "C<use Image::ExifTool::MWG>".
+
+        When reading, the value of each MWG tag is B<Derived From> the specified
+        tags based on the MWG guidelines.  When writing, the appropriate associated
+        tags are written.  The value of the IPTCDigest tag is updated automatically
+        when the IPTC is changed if either the IPTCDigest tag didn't exist
+        beforehand or its value agreed with the original IPTC digest (indicating
+        that the XMP is synchronized with the IPTC).  IPTC information is written
+        only if the original file contained IPTC.
 
         Loading the MWG module activates "strict MWG conformance mode", which has
         the effect of causing EXIF, IPTC and XMP in non-standard locations to be
@@ -50,20 +63,13 @@ sub RecoverTruncatedIPTC($$$);
         records only in the standard location, but writes new tags to any
         EXIF/IPTC/XMP records that exist.
 
-        A problem with the MWG specification is that although XMP:Creator and
-        IPTC:By-line are list-type tags, the corresponding EXIF tag, EXIF:Artist, is
-        not.  So, short of changing the EXIF specification to allow EXIF:Artist to
-        store a list of strings, the only reasonable alternative is to NOT allow
-        MWG:Creator to be written as a list.  (It is insufficient to say that
-        EXIF:Artist may store multiple strings separated with semicolons because
-        there is nothing in any of the MWG, EXIF, XMP or IPTC specifications that
-        states an individual Creator string may not contain a semicolon, and for
-        this to be feasible all of these specifications would require this
-        restriction.)  For this reason, the MWG:Creator tag is written as a single
-        string.  Multiple names may of course be written within the string (and it
-        would be reasonable to separate them with semicolons as suggested by the MWG
-        and EXIF specifications), but this tag is not treated as a list by ExifTool
-        when writing.
+        A complication of the specification is that although the MWG:Creator
+        property may consist of multiple values, the associated EXIF tag
+        (EXIF:Artist) is only a simple string.  To resolve this discrepancy the MWG
+        recommends a technique which allows a list of values to be stored in a
+        string by using a semicolon-space separator (with quotes around values if
+        necessary).  When the MWG module is loaded, ExifTool automatically
+        implements this policy and changes EXIF:Artist to a list-type tag.
     },
     Keywords => {
         Flags  => ['Writable','List'],
@@ -96,7 +102,7 @@ sub RecoverTruncatedIPTC($$$);
             4 => 'IPTCDigest',
         },
         ValueConv => q{
-            return $val[0] if defined $val[0];
+            return $val[0] if defined $val[0] and $val[0] !~ /^ *$/;
             return $val[2] if not defined $val[3] or (defined $val[2] and
                              (not defined $val[4] or $val[3] eq $val[4]));
             return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[1], $val[2], 2000);
@@ -124,7 +130,9 @@ sub RecoverTruncatedIPTC($$$);
             6 => 'IPTCDigest',
         },
         ValueConv => q{
-            return $val[1] ? "$val[0].$val[1]" : $val[0] if defined $val[0];
+            if (defined $val[0] and $val[0] !~ /^[: ]*$/) {
+                return ($val[1] and $val[1] !~ /^ *$/) ? "$val[0].$val[1]" : $val[0];
+            }
             return $val[4] if not defined $val[5] or (defined $val[4] and
                              (not defined $val[6] or $val[5] eq $val[6]));
             return $val[3] ? "$val[2] $val[3]" : $val[2] if $val[2];
@@ -158,7 +166,9 @@ sub RecoverTruncatedIPTC($$$);
             6 => 'IPTCDigest',
         },
         ValueConv => q{
-            return $val[1] ? "$val[0].$val[1]" : $val[0] if defined $val[0];
+            if (defined $val[0] and $val[0] !~ /^[: ]*$/) {
+                return ($val[1] and $val[1] !~ /^ *$/) ? "$val[0].$val[1]" : $val[0];
+            }
             return $val[4] if not defined $val[5] or (defined $val[4] and
                              (not defined $val[6] or $val[5] eq $val[6]));
             return $val[3] ? "$val[2] $val[3]" : $val[2] if $val[2];
@@ -188,7 +198,9 @@ sub RecoverTruncatedIPTC($$$);
             4 => 'IPTCDigest',
         },
         ValueConv => q{
-            return $val[1] ? "$val[0].$val[1]" : $val[0] if defined $val[0];
+            if (defined $val[0] and $val[0] !~ /^[: ]*$/) {
+                return ($val[1] and $val[1] !~ /^ *$/) ? "$val[0].$val[1]" : $val[0];
+            }
             return $val[2] if not defined $val[3] or not defined $val[4] or $val[3] eq $val[4];
             return undef;
         },
@@ -236,7 +248,7 @@ sub RecoverTruncatedIPTC($$$);
             4 => 'IPTCDigest',
         },
         ValueConv => q{
-            return $val[0] if defined $val[0];
+            return $val[0] if defined $val[0] and $val[0] !~ /^ *$/;
             return $val[2] if not defined $val[3] or (defined $val[2] and
                              (not defined $val[4] or $val[3] eq $val[4]));
             return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[1], $val[2], 128);
@@ -251,7 +263,7 @@ sub RecoverTruncatedIPTC($$$);
     },
     Creator => {
         Groups => { 2 => 'Author' },
-        Writable => 1,
+        Flags  => ['Writable','List'],
         Desire => {
             0 => 'EXIF:Artist',
             1 => 'IPTC:By-line', # (32-character limit)
@@ -259,9 +271,8 @@ sub RecoverTruncatedIPTC($$$);
             3 => 'CurrentIPTCDigest',
             4 => 'IPTCDigest',
         },
-        Notes => 'NOT a list-type tag.  See "A problem with the MWG specification" above',
         ValueConv => q{
-            return $val[0] if defined $val[0];
+            return $val[0] if defined $val[0] and $val[0] !~ /^ *$/;
             return $val[2] if not defined $val[3] or (defined $val[2] and
                              (not defined $val[4] or $val[3] eq $val[4]));
             return Image::ExifTool::MWG::RecoverTruncatedIPTC($val[1], $val[2], 32);
@@ -270,8 +281,8 @@ sub RecoverTruncatedIPTC($$$);
         WriteCheck => 'Image::ExifTool::MWG::ReconcileIPTCDigest($self)',
         WriteAlso  => {
             'EXIF:Artist'    => '$val',
-            'IPTC:By-line'   => '$opts{Replace} = 1; $opts{EditGroup} = 1; $val',
-            'XMP-dc:Creator' => '$opts{Replace} = 1; $val',
+            'IPTC:By-line'   => '$opts{EditGroup} = 1; $val',
+            'XMP-dc:Creator' => '$val',
         },
     },
     Country => {
@@ -369,6 +380,103 @@ unless ($Image::ExifTool::documentOnly) {
                                      'Image::ExifTool::Composite');
 }
 
+# modify EXIF:Artist to behave as a list-type tag
+{
+    my $artist = $Image::ExifTool::Exif::Main{0x13b};
+    $$artist{List} = 1;
+    $$artist{IsOverwriting} = \&OverwriteStringList;
+    $$artist{RawConv} = \&StringToList;
+}
+
+#------------------------------------------------------------------------------
+# Change a list of values to a string using MWG rules
+# Inputs: 0)reference to list of values
+# Returns: string of values (and may reformat list entries)
+sub ListToString($)
+{
+    my $vals = shift;
+    foreach (@$vals) {
+        # double all quotes in value and quote the value if it begins
+        # with a quote or contains a semicolon-space separator
+        if (/^"/ or /; /) {
+            s/"/""/g;       # double all quotes
+            $_ = qq{"$_"};  # quote the value
+        }
+    }
+    return join('; ', @$vals);
+}
+
+#------------------------------------------------------------------------------
+# Change a string value to a list of values using MWG rules
+# Inputs: 0) string of values, 1) ExifTool ref
+# Returns: value or list reference if more than one value
+# Notes: Sets Warning tag on error
+sub StringToList($$)
+{
+    my ($str, $exifTool) = @_;
+    my (@vals, $inQuotes);
+    my @t = split '; ', $str, -1;
+    foreach (@t) {
+        my $wasQuotes = $inQuotes;
+        $inQuotes = 1 if not $inQuotes and s/^"//;
+        if ($inQuotes) {
+            # remove the last quote and reset the inQuotes flag if
+            # the value ended in an odd number of quotes
+            $inQuotes = 0 if s/((^|[^"])("")*)"$/$1/;
+            s/""/"/g;   # un-double the contained quotes
+        }
+        if ($wasQuotes) {
+            # previous separator was quoted, so concatinate with previous value
+            $vals[-1] .= '; ' . $_;
+        } else {
+            push @vals, $_;
+        }
+    }
+    $exifTool->Warn('Incorrectly quoted MWG string-list value') if $inQuotes;
+    return @vals > 1 ? \@vals : $vals[0];
+}
+
+#------------------------------------------------------------------------------
+# Handle logic for overwriting EXIF string-type list tag
+# Inputs: 0) new value hash ref, 1) new value hash ref,
+#         2) old string value (or undef if it didn't exist), 3) new value ref
+# Returns: 1 and sets the new value for the tag
+sub OverwriteStringList($$$$)
+{
+    local $_;
+    my ($exifTool, $nvHash, $val, $newValuePt) = @_;
+    my (@new, $delIndex);
+    if ($$nvHash{DelValue} and defined $val) {
+        # preserve specified old values
+        my $old = StringToList($val, $exifTool);
+        my @old = ref $old eq 'ARRAY' ? @$old : $old;
+        if (@{$$nvHash{DelValue}}) {
+            my %del;
+            $del{$_} = 1 foreach @{$$nvHash{DelValue}};
+            foreach (@old) {
+                $del{$_} or push(@new, $_), next;
+                $delIndex or $delIndex = scalar @new;
+            }
+        } else {
+            push @new, @old;
+        }
+    }
+    # add new values (at location of deleted values, if any)
+    if ($$nvHash{Value}) {
+        if (defined $delIndex) {
+            splice @new, $delIndex, 0, @{$$nvHash{Value}};
+        } else {
+            push @new, @{$$nvHash{Value}};
+        }
+    }
+    if (@new) {
+        # convert back to string format
+        $$newValuePt = ListToString(\@new);
+    } else {
+        $$newValuePt = undef;   # delete the tag
+    }
+    return 1;
+}
 
 #------------------------------------------------------------------------------
 # Reconcile IPTC digest after writing an MWG tag
@@ -448,7 +556,7 @@ after loading the MWG module):
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
