@@ -16,6 +16,8 @@
 #               9) http://developers.videolan.org/vlc/vlc/doc/doxygen/html/codecs_8h-source.html
 #              10) http://wiki.multimedia.cx/index.php?title=TwoCC
 #              11) Andreas Winter (SCLive) private communication
+#              12) http://abcavi.kibi.ru/infotags.htm
+#              13) http://tech.ebu.ch/docs/tech/tech3285.pdf
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::RIFF;
@@ -24,9 +26,9 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.20';
+$VERSION = '1.30';
 
-sub ConvertTimeCode($);
+sub ConvertTimecode($);
 
 # type of current stream
 $Image::ExifTool::RIFF::streamType = '';
@@ -282,17 +284,26 @@ $Image::ExifTool::RIFF::streamType = '';
 %Image::ExifTool::RIFF::Main = (
     PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessChunks,
     NOTES => q{
-        Windows WAV and AVI files are RIFF format files.  According to the EXIF
-        specification, Meta information is embedded in two types of RIFF C<LIST>
-        chunks: C<INFO> and C<exif>, and information about the audio content is
-        stored in the C<fmt > chunk.  As well as this information, some video
-        information and proprietary manufacturer-specific information is also
-        extracted.
+        The RIFF container format is used various types of fines including WAV, AVI
+        and WEBP.  According to the EXIF specification, Meta information is embedded
+        in two types of RIFF C<LIST> chunks: C<INFO> and C<exif>, and information
+        about the audio content is stored in the C<fmt > chunk.  As well as this
+        information, some video information and proprietary manufacturer-specific
+        information is also extracted.
+        
+        Large AVI videos may be a concatenation of two or more RIFF chunks.
+        For these files, information is extracted from subsequent RIFF
+        chunks as sub-documents, but the Duration is calculated for the full
+        video.
     },
    'fmt ' => {
         Name => 'AudioFormat',
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::AudioFormat' },
     },
+   'bext' => {
+        Name => 'BroadcastExtension',
+        SubDirectory => { TagTable => 'Image::ExifTool::RIFF::BroadcastExt' },
+    },
     LIST_INFO => {
         Name => 'Info',
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::Info' },
@@ -352,9 +363,14 @@ $Image::ExifTool::RIFF::streamType = '';
             },
         },
         {
-            Name => 'UnknownJunk',
-            # try to interpret unknown junk as a string
-            RawConv => '$_=$val; /^[^\0-\x1f\x7f-\xff]+$/ ? $_ : undef',
+            Name => 'PentaxJunk', # (Optio RS1000)
+            Condition => '$$valPt =~ /^IIII\x01\0/',
+            SubDirectory => { TagTable => 'Image::ExifTool::Pentax::Junk' },
+        },
+        {
+            Name => 'TextJunk',
+            # try to interpret unknown junk as an ASCII string
+            RawConv => '$val =~ /^([^\0-\x1f\x7f-\xff]+)\0*$/ ? $1 : undef',
         }
     ],
     _PMX => { #PH (Adobe CS3 Bridge)
@@ -366,6 +382,10 @@ $Image::ExifTool::RIFF::streamType = '';
         Name => 'OldXMP',
         Binary => 1,
     },
+    olym => {
+        Name => 'Olym',
+        SubDirectory => { TagTable => 'Image::ExifTool::Olympus::WAV' },
+    },
 );
 
 # the maker notes used by some digital cameras
@@ -399,15 +419,61 @@ $Image::ExifTool::RIFF::streamType = '';
     7 => 'BitsPerSample',
 );
 
+# Broadcast Audio Extension 'bext' information (ref 13)
+%Image::ExifTool::RIFF::BroadcastExt = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Audio' },
+    NOTES => q{
+        Information found in the Broadcast Audio Extension chunk (see
+        L<http://tech.ebu.ch/docs/tech/tech3285.pdf>).
+    },
+    0 => {
+        Name => 'Description',
+        Format => 'string[256]',
+    },
+    256 => {
+        Name => 'Originator',
+        Format => 'string[32]',
+    },
+    288 => {
+        Name => 'OriginatorReference',
+        Format => 'string[32]',
+    },
+    320 => {
+        Name => 'DateTimeOriginal',
+        Description => 'Date/Time Original',
+        Groups => { 2 => 'Time' },
+        Format => 'string[18]',
+        ValueConv => '$_=$val; tr/-/:/; s/^(\d{4}:\d{2}:\d{2})/$1 /; $_',
+        PrintConv => '$self->ConvertDateTime($val)',
+    },
+    338 => {
+        Name => 'TimeReference',
+        Notes => 'first sample count since midnight',
+        Format => 'int32u[2]',
+        ValueConv => 'my @v=split(" ",$val); $v[0] + $v[1] * 4294967296',
+    },
+    346 => {
+        Name => 'BWFVersion',
+        Format => 'int16u',
+    },
+    # 348 - int8u[64] - SMPTE 330M UMID (Unique Material Identifier)
+    # 412 - int8u[190] - reserved
+    602 => {
+        Name => 'CodingHistory',
+        Format => 'string[$size-602]',
+    },
+);
+
 # Sub chunks of INFO LIST chunk
 %Image::ExifTool::RIFF::Info = (
     PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessChunks,
     GROUPS => { 2 => 'Audio' },
     NOTES => q{
         RIFF INFO tags found in WAV audio and AVI video files.  Tags which are part
-        of the EXIF 2.2 specification (Tag ID's beginning with "I") have an
-        underlined Tag Name in the HTML version of this documentation.  Other tags
-        are found in AVI files generated by Sony Vegas video editing software.
+        of the EXIF 2.3 specification have an underlined Tag Name in the HTML
+        version of this documentation.  Other tags are found in AVI files generated
+        by some software.
     },
     IARL => 'ArchivalLocation',
     IART => { Name => 'Artist',    Groups => { 2 => 'Author' } },
@@ -441,24 +507,83 @@ $Image::ExifTool::RIFF::streamType = '';
     ISRC => 'Source',
     ISRF => 'SourceForm',
     ITCH => 'Technician',
-    # the following tags (TAPE->DTIM) are Sony Vegas AVI tags, and are
-    # also used by SCLive and Adobe Premier (ref 11)
+#
+# 3rd party tags
+#
+    # internet movie database (ref 12)
+    ISGN => 'SecondaryGenre',
+    IWRI => 'WrittenBy',
+    IPRO => 'ProducedBy',
+    ICNM => 'Cinematographer',
+    IPDS => 'ProductionDesigner',
+    IEDT => 'EditedBy',
+    ICDS => 'CostumeDesigner',
+    IMUS => 'MusicBy',
+    ISTD => 'ProductionStudio',
+    IDST => 'DistributedBy',
+    ICNT => 'Country',
+    ILNG => 'Language',
+    IRTD => 'Rating',
+    ISTR => 'Starring',
+    # MovieID (ref12)
+    TITL => 'Title',
+    DIRC => 'Directory',
+    YEAR => 'Year',
+    GENR => 'Genre',
+    COMM => 'Comments',
+    LANG => 'Language',
+    AGES => 'Rated',
+    STAR => 'Starring',
+    CODE => 'EncodedBy',
+    PRT1 => 'Part',
+    PRT2 => 'NumberOfParts',
+    # Morgan Multimedia INFO tags (ref 12)
+    IAS1 => 'FirstLanguage',
+    IAS2 => 'SecondLanguage',
+    IAS3 => 'ThirdLanguage',
+    IAS4 => 'FourthLanguage',
+    IAS5 => 'FifthLanguage',
+    IAS6 => 'SixthLanguage',
+    IAS7 => 'SeventhLanguage',
+    IAS8 => 'EighthLanguage',
+    IAS9 => 'NinthLanguage',
+    ICAS => 'DefaultAudioStream',
+    IBSU => 'BaseURL',
+    ILGU => 'LogoURL',
+    ILIU => 'LogoIconURL',
+    IWMU => 'WatermarkURL',
+    IMIU => 'MoreInfoURL',
+    IMBI => 'MoreInfoBannerImage',
+    IMBU => 'MoreInfoBannerURL',
+    IMIT => 'MoreInfoText',
+    # GSpot INFO tags (ref 12)
+    IENC => 'EncodedBy',
+    IRIP => 'RippedBy',
+    # Sound Forge Pro tags
+    DISP => 'SoundSchemeTitle',
+    TLEN => { Name => 'Length', ValueConv => '$val/1000', PrintConv => '"$val s"' },
+    TRCK => 'TrackNumber',
+    TURL => 'URL',
+    TVER => 'Version',
+    LOCA => 'Location',
+    TORG => 'Organization',
+    # Sony Vegas AVI tags, also used by SCLive and Adobe Premier (ref 11)
     TAPE => {
         Name => 'TapeName',
         Groups => { 2 => 'Video' },
     },
     TCOD => {
-        Name => 'StartTimeCode',
+        Name => 'StartTimecode',
         # this is the tape time code for the start of the video
         Groups => { 2 => 'Video' },
         ValueConv => '$val * 1e-7',
-        PrintConv => \&ConvertTimeCode,
+        PrintConv => \&ConvertTimecode,
     },
     TCDO => {
-        Name => 'EndTimeCode',
+        Name => 'EndTimecode',
         Groups => { 2 => 'Video' },
         ValueConv => '$val * 1e-7',
-        PrintConv => \&ConvertTimeCode,
+        PrintConv => \&ConvertTimecode,
     },
     VMAJ => {
         Name => 'VegasVersionMajor',
@@ -494,6 +619,8 @@ $Image::ExifTool::RIFF::streamType = '';
         ValueConv => q{
             my @v = split ' ', $val;
             return undef unless @v == 2;
+            # the Kodak EASYSHARE Sport stores this incorrectly as a string:
+            return $val if $val =~ /^\d{4}:\d{2}:\d{2} \d{2}:\d{2}:\d{2}$/;
             # get time in seconds
             $val = 1e-7 * ($v[0] * 4294967296 + $v[1]);
             # shift from Jan 1, 1601 to Jan 1, 1970
@@ -508,7 +635,7 @@ $Image::ExifTool::RIFF::streamType = '';
 %Image::ExifTool::RIFF::Exif = (
     PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessChunks,
     GROUPS => { 2 => 'Audio' },
-    NOTES => 'These tags are part of the EXIF 2.2 specification for WAV audio files.',
+    NOTES => 'These tags are part of the EXIF 2.3 specification for WAV audio files.',
     ever => 'ExifVersion',
     erel => 'RelatedImageFile',
     etim => { Name => 'TimeCreated', Groups => { 2 => 'Time' } },
@@ -536,7 +663,7 @@ $Image::ExifTool::RIFF::streamType = '';
         ValueConv => 'Image::ExifTool::RIFF::ConvertRIFFDate($val)',
         PrintConv => '$self->ConvertDateTime($val)',
     },
-    ISMP => 'TimeCode',
+    ISMP => 'Timecode',
     LIST_strl => {
         Name => 'Stream',
         SubDirectory => { TagTable => 'Image::ExifTool::RIFF::Stream' },
@@ -611,10 +738,18 @@ $Image::ExifTool::RIFF::streamType = '';
     GROUPS => { 2 => 'Video' },
     dmlh => {
         Name => 'ExtendedAVIHeader',
-        Flags => ['Binary','Unknown'],
+        SubDirectory => { TagTable => 'Image::ExifTool::RIFF::ExtAVIHdr' },
     },
 );
 
+# Extended AVI Header tags (ref http://www.morgan-multimedia.com/download/odmlff2.pdf)
+%Image::ExifTool::RIFF::ExtAVIHdr = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Video' },
+    FORMAT => 'int32u',
+    0 => 'TotalFrameCount',
+);
+
 %Image::ExifTool::RIFF::StreamHeader = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     GROUPS => { 2 => 'Video' },
@@ -666,13 +801,13 @@ $Image::ExifTool::RIFF::streamType = '';
             Format => 'rational64u',
             # (must use RawConv because raw value used in Composite tag)
             RawConv => '$val ? 1/$val : undef',
-            PrintConv => 'int($val * 100 + 0.5) / 100',
+            PrintConv => 'int($val * 1000 + 0.5) / 1000',
         },
         {
             Name => 'StreamSampleRate',
             Format => 'rational64u',
             ValueConv => '$val ? 1/$val : 0',
-            PrintConv => 'int($val * 100 + 0.5) / 100',
+            PrintConv => 'int($val * 1000 + 0.5) / 1000',
         },
     ],
   # 7 => 'Start',
@@ -704,7 +839,10 @@ $Image::ExifTool::RIFF::streamType = '';
 %Image::ExifTool::RIFF::StreamData = ( #PH
     PROCESS_PROC => \&Image::ExifTool::RIFF::ProcessStreamData,
     GROUPS => { 2 => 'Video' },
-    NOTES => 'This chunk contains EXIF information in FujiFilm AVI videos.',
+    NOTES => q{
+        This chunk is used to store proprietary information in AVI videos from some
+        cameras.  The first 4 characters of the data are used as the Tag ID below.
+    },
     AVIF => {
         Name => 'AVIF',
         SubDirectory => {
@@ -718,6 +856,7 @@ $Image::ExifTool::RIFF::streamType = '';
         Name => 'CasioData',
         SubDirectory => { TagTable => 'Image::ExifTool::Casio::AVI' },
     },
+    Zora => 'VendorName',   # (Samsung PL90 AVI files)
     unknown => {
         Name => 'UnknownData',
         # try to interpret unknown stream data as a string
@@ -729,40 +868,29 @@ $Image::ExifTool::RIFF::streamType = '';
 %Image::ExifTool::RIFF::Composite = (
     Duration => {
         Require => {
-            0 => 'FrameRate',
-            1 => 'FrameCount',
+            0 => 'RIFF:FrameRate',
+            1 => 'RIFF:FrameCount',
         },
         Desire => {
             2 => 'VideoFrameRate',
             3 => 'VideoFrameCount',
         },
-        # this is annoying.  Apparently (although I couldn't verify this), FrameCount
-        # in the RIFF header includes multiple video tracks if they exist (ie. with the
-        # FujiFilm REAL 3D AVI's), but the video stream information isn't reliable for
-        # some cameras (ie. Olympus FE models), so use the video stream information
-        # only if the RIFF header duration is 2 to 3 times longer
-        RawConv => q {
-            my ($dur1, $dur2);
-            $dur1 = $val[1] / $val[0] if $val[0];
-            return $dur1 unless $val[2] and $val[3];
-            $dur2 = $val[3] / $val[2];
-            my $rat = $dur1 / $dur2;
-            return ($rat > 1.9 and $rat < 3.1) ? $dur2 : $dur1;
-        },
+        RawConv => 'Image::ExifTool::RIFF::CalcDuration($self, @val)',
         PrintConv => 'ConvertDuration($val)',
     },
     Duration2 => {
         Name => 'Duration',
         Require => {
-            0 => 'AvgBytesPerSec',
+            0 => 'RIFF:AvgBytesPerSec',
             1 => 'FileSize',
         },
         Desire => {
             # check FrameCount because this calculation only applies
             # to audio-only files (ie. WAV)
             2 => 'FrameCount',
+            3 => 'VideoFrameCount',
         },
-        RawConv => '($val[0] and not $val[2]) ? $val[1] / $val[0] : undef',
+        RawConv => '($val[0] and not ($val[2] or $val[3])) ? $val[1] / $val[0] : undef',
         PrintConv => 'ConvertDuration($val)',
     },
 );
@@ -798,7 +926,7 @@ sub ConvertRIFFDate($)
 # Print time
 # Inputs: 0) time in seconds
 # Returns: time string
-sub ConvertTimeCode($)
+sub ConvertTimecode($)
 {
     my $val = shift;
     my $hr = int($val / 3600);
@@ -808,6 +936,59 @@ sub ConvertTimeCode($)
     return sprintf("%d:%.2d:%05.2f", $hr, $min, $val);
 }
 
+#------------------------------------------------------------------------------
+# Calculate duration of RIFF
+# Inputs: 0) ExifTool ref, 1/2) RIFF:FrameRate/Count, 2/3) VideoFrameRate/Count
+# Returns: Duration in seconds or undef
+# Notes: Sums duration of all sub-documents (concatenated AVI files)
+sub CalcDuration($@)
+{
+    my ($exifTool, @val) = @_;
+    my $totalDuration = 0;
+    my $subDoc = 0;
+    my @keyList;
+    for (;;) {
+        # this is annoying.  Apparently (although I couldn't verify this), FrameCount
+        # in the RIFF header includes multiple video tracks if they exist (ie. with the
+        # FujiFilm REAL 3D AVI's), but the video stream information isn't reliable for
+        # some cameras (ie. Olympus FE models), so use the video stream information
+        # only if the RIFF header duration is 2 to 3 times longer
+        my $dur1 = $val[1] / $val[0] if $val[0];
+        if ($val[2] and $val[3]) {
+            my $dur2 = $val[3] / $val[2];
+            my $rat = $dur1 / $dur2;
+            $dur1 = $dur2 if $rat > 1.9 and $rat < 3.1;
+        }
+        $totalDuration += $dur1 if defined $dur1;
+        last unless $subDoc++ < $$exifTool{DOC_COUNT};
+        # get tag values for next sub-document
+        my @tags = qw(FrameRate FrameCount VideoFrameRate VideoFrameCount);
+        my $rawValue = $$exifTool{VALUE};
+        my ($i, $j, $key, $keys);
+        for ($i=0; $i<@tags; ++$i) {
+            if ($subDoc == 1) {
+                # generate list of available keys for each tag
+                $keys = $keyList[$i] = [ ];
+                for ($j=0; ; ++$j) {
+                    $key = $tags[$i];
+                    $key .= " ($j)" if $j;
+                    last unless defined $$rawValue{$key};
+                    push @$keys, $key;
+                }
+            } else {
+                $keys = $keyList[$i];
+            }
+            # find key for tag in this sub-document
+            my $grp = "Doc$subDoc";
+            $grp .= ":RIFF" if $i < 2; # (tags 0 and 1 also in RIFF group)
+            $key = $exifTool->GroupMatches($grp, $keys);
+            $val[$i] = $key ? $$rawValue{$key} : undef;
+        }
+        last unless defined $val[0] and defined $val[1]; # (Require'd tags)
+    }
+    return $totalDuration;
+}
+
 #------------------------------------------------------------------------------
 # Process stream data
 # Inputs: 0) ExifTool object ref, 1) dirInfo reference, 2) tag table ref
@@ -936,13 +1117,13 @@ sub ProcessRIFF($$)
     my ($exifTool, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
     my ($buff, $err);
-    my %types = ( 'WAVE' => 'WAV', 'AVI ' => 'AVI' );
+    my %types = ( 'WAVE' => 'WAV', 'AVI ' => 'AVI', 'WEBP' => 'WEBP' );
     my $verbose = $exifTool->Options('Verbose');
 
     # verify this is a valid RIFF file
     return 0 unless $raf->Read($buff, 12) == 12;
     return 0 unless $buff =~ /^RIFF....(.{4})/s;
-    $exifTool->SetFileType($types{$1}); # set type to 'WAV', 'AVI' or 'RIFF'
+    $exifTool->SetFileType($types{$1}); # set type to 'WAV', 'AVI', 'WEBP' or 'RIFF'
     $Image::ExifTool::RIFF::streamType = '';    # initialize stream type
     SetByteOrder('II');
     my $tagTablePtr = GetTagTable('Image::ExifTool::RIFF::Main');
@@ -977,7 +1158,7 @@ sub ProcessRIFF($$)
         }
         # RIFF chunks are padded to an even number of bytes
         my $len2 = $len + ($len & 0x01);
-        if ($$tagTablePtr{$tag} or ($verbose and $tag !~ /^(data|idx1|LIST_movi)$/)) {
+        if ($$tagTablePtr{$tag} or ($verbose and $tag !~ /^(data|idx1|LIST_movi|RIFF)$/)) {
             $raf->Read($buff, $len2) == $len2 or $err=1, last;
             $exifTool->HandleTag($tagTablePtr, $tag, $buff,
                 DataPt  => \$buff,
@@ -986,11 +1167,17 @@ sub ProcessRIFF($$)
                 Size    => $len2,
                 Base    => $pos,
             );
+        } elsif ($tag eq 'RIFF') {
+            # don't read into RIFF chunk (ie. concatenated video file)
+            $raf->Read($buff, 4) == 4 or $err=1, last;
+            # extract information from remaining file as an embedded file
+            $$exifTool{DOC_NUM} = ++$$exifTool{DOC_COUNT}
         } else {
             $raf->Seek($len2, 1) or $err=1, last;
         }
         $pos += $len2;
     }
+    delete $$exifTool{DOC_NUM};
     $err and $exifTool->Warn('Error reading RIFF file (corrupted?)');
     return 1;
 }
@@ -1015,7 +1202,7 @@ including Windows WAV audio and AVI video files.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
